#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/lib/SGVector.h>
#include <shogun/evaluation/ContingencyTableEvaluation.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/io/CSVFile.h>
#include <shogun/neuralnets/NeuralLayers.h>
#include <shogun/labels/BinaryLabels.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/neuralnets/NeuralNetwork.h>
#include <shogun/lib/DynamicObjectArray.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/mathematics/Math.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_labels_test.dat");

CMath::init_random(1);

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CBinaryLabels>(f_labels_train);
auto labels_test = some<CBinaryLabels>(f_labels_test);
//![create_features]

//![add_layers]
auto num_feats = features_train->get_num_features();
auto layers = some<CNeuralLayers>();
layers = layers->input(num_feats);
layers = layers->rectified_linear(10);
layers = layers->softmax(2);
auto all_layers = layers->done();
//![add_layers]

//![create_instance]
auto network = some<CNeuralNetwork>(all_layers);
network->quick_connect();
network->initialize_neural_network();
//![create_instance]

//![set_parameters]
network->set_l2_coefficient(0.01);
network->set_dropout_hidden(0.5);
network->set_max_num_epochs(50);
network->set_gd_mini_batch_size(num_feats);
network->set_gd_learning_rate(0.1);
network->set_gd_momentum(0.9);
//![set_parameters]

//![train_and_apply]
network->set_labels(labels_train);
network->train(features_train);
auto labels_predict = network->apply_binary(features_test);
//![train_and_apply]

//![get_params]
auto parameters = network->get_parameters();
//![get_params]

//![evaluate_accuracy]
auto am = some<CAccuracyMeasure>();
auto accuracy = am->evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// additional integration testing variables
auto output = labels_predict->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("feedforward_net_classification.dat", 'w');
__sg_storage->append_wrapped_int(num_feats, "num_feats");
__sg_storage->append_wrapped_real_vector(parameters, "parameters");
__sg_storage->append_wrapped_real(accuracy, "accuracy");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
